@@ -1,5 +1,5 @@
 /*
-  $Id: Configuration.h 13840 2012-01-02 13:47:26Z sloot $
+  $Id: Configuration.h 14344 2012-02-27 16:40:05Z sloot $
   $URL: https://ilk.uvt.nl/svn/sources/Frog/trunk/include/frog/Configuration.h $
 
   Copyright (c) 2006 - 2012
@@ -35,7 +35,11 @@ class Configuration {
 public:
   bool fill( const std::string& );
   bool fill( const std::string&, const std::string& );
+  bool hasSection( const std::string& ) const;
   std::string lookUp( const std::string&, const std::string& = "" ) const;
+  std::string set( const std::string&, 
+		   const std::string&, 
+		   const std::string& = "" );
   void dump( std::ostream& );
   std::string configDir() const { return cdir; };
 private:
