@@ -1,5 +1,5 @@
 /*
-  $Id: Frog.h 13840 2012-01-02 13:47:26Z sloot $
+  $Id: Frog.h 14344 2012-02-27 16:40:05Z sloot $
   $URL: https://ilk.uvt.nl/svn/sources/Frog/trunk/include/frog/Frog.h $
 
   Copyright (c) 2006 - 2012
@@ -27,10 +27,8 @@
 */                                                                   
 
 
-#ifndef __DEMO_OPTIONS__
-#define __DEMO_OPTIONS__
-
-#define MAX_NAMELEN 2048 
+#ifndef FROG_H
+#define FROG_H
 
 #include <set>
 #include "timbl/LogStream.h"
@@ -40,15 +38,12 @@
 std::string prefix( const std::string&, const std::string& );
 bool existsDir( const std::string& );
 std::string& trim(std::string &str);
-std::string escape( const std::string& );
 
 void getFileNames( const std::string&, std::set<std::string>& );
 
 extern LogStream *theErrLog;
 
 extern int tpDebug;
-extern bool keepIntermediateFiles;
-extern bool doServer;
 extern Configuration configuration;
 
 class TimerBlock{
@@ -59,11 +54,14 @@ public:
   Common::Timer mbmaTimer;
   Common::Timer mwuTimer;
   Common::Timer tagTimer;
+  Common::Timer iobTimer;
+  Common::Timer nerTimer;
   Common::Timer prepareTimer;
   Common::Timer pairsTimer;
   Common::Timer relsTimer;
   Common::Timer dirTimer;
   Common::Timer csiTimer;
+  Common::Timer frogTimer;
 };
 
 #endif
