@@ -1,5 +1,5 @@
 /*
-  $Id: Parser.h 13840 2012-01-02 13:47:26Z sloot $
+  $Id: Parser.h 14447 2012-03-12 16:34:56Z sloot $
   $URL: https://ilk.uvt.nl/svn/sources/Frog/trunk/include/frog/Parser.h $
 
   Copyright (c) 2006 - 2012
@@ -80,11 +80,14 @@ struct parseData;
 
 class Parser {
  public:
- Parser():pairs(0),dir(0),rels(0),PI(0),isInit(false){};
+ Parser():pairs(0),dir(0),rels(0),PI(0),isInit(false),keepIntermediate(false){};
   ~Parser();
   bool init( const Configuration& );
-  void Parse( folia::FoliaElement *, const std::string&, TimerBlock& );
-  void prepareParse( folia::FoliaElement *, parseData& );
+  void addDeclaration( folia::Document& doc ) const;
+  void Parse( folia::Sentence *, 
+	      const std::string&, const std::string&, TimerBlock& );
+  void prepareParse( folia::Sentence *, 
+		     const std::string&, parseData& );
   void createParserFile( const parseData& );
  private:
   void createRelDir( const parseData& );
@@ -98,6 +101,10 @@ class Parser {
   bool isInit;
   std::string fileName;
   LogStream *parseLog;
+  bool keepIntermediate;
+  std::string version;
+  std::string tagset;
+
 };
 
 
