@@ -1,6 +1,6 @@
 /*
-  $Id: $
-  $URL: $
+  $Id: cgn_tagger_mod.h 14489 2012-03-22 11:37:02Z sloot $
+  $URL: https://ilk.uvt.nl/svn/sources/Frog/trunk/include/frog/cgn_tagger_mod.h $
 
   Copyright (c) 2006 - 2012
   Tilburg University
@@ -26,19 +26,26 @@
       timbl@uvt.nl
 */
 
-#ifndef TAGGER_MOD_H
-#define TAGGER_MOD_H
+#ifndef CGN_TAGGER_MOD_H
+#define CGN_TAGGER_MOD_H
 
 class Configuration;
 
-class MBTagger {
+class CGNTagger {
  public:
-  MBTagger();
-  ~MBTagger();
+  CGNTagger();
+  ~CGNTagger();
   bool init( const Configuration& );
-  std::string Classify( folia::FoliaElement * );
+  void addDeclaration( folia::Document& ) const;
+  void addTag( folia::Word *, const std::string&, double );
+  void Classify( folia::Sentence * );
+  std::string getTagset() const { return tagset; };
  private:
   MbtAPI *tagger;  
+  LogStream *cgnLog;
+  int debug;
+  std::string tagset;
+  std::string version;
 };
 
-#endif // TAGGER_MOD_H
+#endif // CGN_TAGGER_MOD_H
