@@ -1,5 +1,5 @@
 /*
-  $Id: mblem_mod.h 13840 2012-01-02 13:47:26Z sloot $
+  $Id: mblem_mod.h 14505 2012-03-22 15:33:15Z sloot $
   $URL: https://ilk.uvt.nl/svn/sources/Frog/trunk/include/frog/mblem_mod.h $
 
   Copyright (c) 2006 - 2012
@@ -49,19 +49,25 @@ class Mblem {
   Mblem();
   ~Mblem();
   bool init( const Configuration& );
-  std::string Classify( folia::FoliaElement * );
+  void addDeclaration( folia::Document& doc ) const;
+  std::string Classify( folia::Word * );
  private:
   void read_transtable( const std::string& );
   void create_MBlem_defaults();
   bool readsettings( const std::string& dir, const std::string& fname );
+  void addLemma( folia::FoliaElement *, const std::string&);
+  void addAltLemma( folia::Word *, const std::string&);
   std::string make_instance( const UnicodeString& in );
-  std::string postprocess( folia::FoliaElement * );
+  std::string postprocess( folia::Word * );
   Timbl::TimblAPI *myLex;
   std::string punctuation;
   size_t history;
   int debug;
   std::map <std::string,std::string> classMap;
   std::vector<mblemData> mblemResult;
+  std::string version;
+  std::string tagset;
+  LogStream *mblemLog;
 };
 
 #endif
