@@ -1,5 +1,5 @@
 /*
-  $Id: mbma_mod.h 13948 2012-01-11 10:44:24Z sloot $
+  $Id: mbma_mod.h 14478 2012-03-19 16:10:38Z sloot $
   $URL: https://ilk.uvt.nl/svn/sources/Frog/trunk/include/frog/mbma_mod.h $
 
   Copyright (c) 2006 - 2012
@@ -26,8 +26,8 @@
       timbl@uvt.nl
 */
                                     
-#ifndef __MDMA_MOD__
-#define __MBMA_MOD__
+#ifndef MBMA_MOD_H
+#define MBMA_MOD_H
 
 class MBMAana;
 
@@ -45,9 +45,10 @@ class Configuration;
 class Mbma {
  public:
   Mbma();
-  ~Mbma() { cleanUp(); };
+  ~Mbma();
   bool init( const Configuration& );
-  bool Classify( folia::FoliaElement * );
+  void addDeclaration( folia::Document& doc ) const;
+  bool Classify( folia::Word * );
   std::string getIname( char ) const;
  private:
   void cleanUp();
@@ -69,17 +70,18 @@ class Mbma {
   MBMAana inflectAndAffix( const std::vector<waStruct>& ana );
   void execute( const UnicodeString& word, 
 		const std::vector<std::string>& classes );
-  void postprocess( folia::FoliaElement *, folia::PosAnnotation * );
+  void postprocess( folia::Word *, folia::PosAnnotation * );
   int debugFlag;
-  folia::FoliaElement *createFeature( char c ) const;
-  void addMorph( folia::FoliaElement *, 
-		 const std::vector<std::string>& );
+  void addMorph( folia::Word *, const std::vector<std::string>& );
   std::string MTreeFilename;
   Timbl::TimblAPI *MTree;
   std::map<char,std::string> iNames;  
   std::map<std::string,std::string> tagNames;  
   std::map<std::string,std::string> TAGconv;
   std::vector<MBMAana> analysis;
+  std::string version;
+  std::string tagset;
+  LogStream *mbmaLog;
 };
 
 class MBMAana {
@@ -114,7 +116,7 @@ class MBMAana {
   std::string tag;
   std::string infl;
   std::vector<std::string> morphemes;
-  std::string description;  
+  std::string description; 
 };
 
 #endif
