@@ -1,5 +1,5 @@
 /*
-  $Id: Frog.cxx 13962 2012-01-11 14:49:30Z sloot $
+  $Id: Frog.cxx 14574 2012-03-28 13:38:44Z sloot $
   $URL: https://ilk.uvt.nl/svn/sources/Frog/trunk/src/Frog.cxx $
 
   Copyright (c) 2006 - 2012
@@ -52,18 +52,18 @@
 #include "frog/Configuration.h"
 #include "libfolia/document.h"
 #include "libfolia/folia.h"
-#include "ucto/tokenize.h"
+#include "frog/ucto_tokenizer_mod.h"
 #include "frog/mbma_mod.h"
 #include "frog/mblem_mod.h"
 #include "frog/mwu_chunker_mod.h"
 #include "frog/cgn_tagger_mod.h"
+#include "frog/iob_tagger_mod.h"
+#include "frog/ner_tagger_mod.h"
 #include "frog/Parser.h"
 
 using namespace std;
 using namespace folia;
 
-Tokenizer::TokenizerClass tokenizer;
-
 LogStream my_default_log( cerr, "frog-", StampMessage ); // fall-back
 LogStream *theErrLog = &my_default_log;  // fill the externals
 
@@ -71,24 +71,27 @@ string TestFileName;
 string testDirName;
 string tmpDirName;
 string outputFileName;
+string docid = "untitled";
 bool wantOUT;
 string XMLoutFileName;
-bool wantXML;
+bool doXMLin;
+bool doXMLout;
 string outputDirName;
 string xmlDirName;
 set<string> fileNames;
 string ProgName;
 int tpDebug = 0; //0 for none, more for more output
-string sep = " "; // "&= " for cgi 
+unsigned int maxParserTokens = 0; // 0 for unlimited
 bool doTok = true;
 bool doMwu = true;
+bool doIOB = true;
+bool doNER = true;
 bool doParse = true;
 bool doDirTest = false;
 bool doServer = false;
 bool doSentencePerLine = false;
 string listenport = "void";
 string encoding;
-bool keepIntermediateFiles = false;
 
 /* assumptions:
    each components gets its own configfile per cmdline options
@@ -110,32 +113,37 @@ bool keepIntermediateFiles = false;
 */
 
 Configuration configuration;
-static string configFileName = string(SYSCONF_PATH) + '/' + PACKAGE + '/' + "frog.cfg";
+static string configDir = string(SYSCONF_PATH) + "/" + PACKAGE + "/";
+static string configFileName = configDir + "frog.cfg";
 
 void usage( ) {
   cout << endl << "Options:\n";
   cout << "\t============= INPUT MODE (mandatory, choose one) ========================\n"
        << "\t -e <encoding>          specify encoding of the input (default UTF-8)\n"
        << "\t -t <testfile>          Run frog on this file\n"
-       << "\t -S <port>              Run as server instead of reading from testfile)\n"
-       << "\t --testdir=<directory>  All files in this dir will be tested)\n"
+       << "\t -x <testfile>          Run frog on this FoLiA XML file\n"
+       << "\t -S <port>              Run as server instead of reading from testfile\n"
+       << "\t --testdir=<directory>  All files in this dir will be tested\n"
        << "\t -n                     Assume input file to hold one sentence per line\n"
+       << "\t --max-parser-tokens=<n> inhibit parsing when a sentence contains over 'n' tokens. (default: no limit)\n"
+
        << "\t============= MODULE SELECTION ==========================================\n"
-       << "\t --skip=[mpt]  Skip Tokenizer (t), Multi-Word Units (m) or Parser (p) \n"
+       << "\t --skip=[mptnc]    Skip Tokenizer (t), Chunker (c), Multi-Word Units (m), Named Entity Recognition (n), or Parser (p) \n"
        << "\t============= CONFIGURATION OPTIONS =====================================\n"
        << "\t -c <filename>    Set configuration file (default " << configFileName << ")\n"
-       << "\t============= OUTPUT OPTIONS ============================================\n"
-       << "\t --tmpdir=<directory> (location to store intermediate files. Default /tmp )\n"
-       << "\t -o <outputfile>	      Output to file, instead of default stdout\n"
-       << "\t -X <xmlfile>      Output also to an XML file in folia format\n"
-       << "\t --outputdir=<dir> Output to dir, instead of default stdout\n"
-       << "\t --xmldir=<dir> Use 'dir' to output FoliA XML to.\n"      
-       << "\t --keep-parser-files=[yes|no] keep intermediate parser files, (last sentence only)\n"
+       << "\t============= OUTPUT OPTIONS ============================================\n"      
+       << "\t -o <outputfile>	    Output columned output to file, instead of default stdout\n"
+       << "\t -X <xmlfile>          Output also to an XML file in FoLiA format\n"
+       << "\t --id=<docid>          Document ID, used in FoLiA output. (Default 'untitled')\n"
+       << "\t --outputdir=<dir>     Output to dir, instead of default stdout\n"
+       << "\t --xmldir=<dir>        Use 'dir' to output FoliA XML to.\n"
+       << "\t --tmpdir=<directory>  (location to store intermediate files. Default /tmp )\n"      
+       << "\t --keep-parser-files=[yes|no] keep intermediate parser files. (last sentence only).\n"
        << "\t============= OTHER OPTIONS ============================================\n"
        << "\t -h. give some help.\n"
-       << "\t -V or --version . Show version info.\n"
-       << "\t -d <debug level> (for more verbosity)\n"
-       << "\t -Q               Enable quote detection in tokeniser\n";
+       << "\t -V or --version .   Show version info.\n"
+       << "\t -d <debug level>    (for more verbosity)\n"
+       << "\t -Q                   Enable quote detection in tokeniser.\n";
 }
 
 //**** stuff to process commandline options *********************************
@@ -144,14 +152,18 @@ static Mbma myMbma;
 static Mblem myMblem;
 static Mwu myMwu;
 static Parser myParser;
-static MBTagger myCGNTagger;
+static CGNTagger myCGNTagger;
+static IOBTagger myIOBTagger;
+static NERTagger myNERTagger;
+static UctoTokenizer tokenizer;
+
 
 bool parse_args( TimblOpts& Opts ) {
   string value;
   bool mood;
   if ( Opts.Find('V', value, mood ) ||
        Opts.Find("version", value, mood ) ){
-    // we alreade did that
+    // we already did show what we wanted.
     exit( EXIT_SUCCESS );
   }
   if ( Opts.Find ('h', value, mood)) {
@@ -166,16 +178,19 @@ bool parse_args( TimblOpts& Opts ) {
 
   if ( configuration.fill( configFileName ) ){
     cerr << "config read from: " << configFileName << endl;
-    //    configuration.dump( cout );
   }
   else {
     cerr << "failed te read configuration from! '" << configFileName << "'" << endl;
+    cerr << "did you correctly install the frogdata package?" << endl;
     return false;
   }
 
   // debug opts
   if ( Opts.Find ('d', value, mood)) {
-    tpDebug = atoi(value.c_str());
+    if ( !Timbl::stringTo<int>( value, tpDebug ) ){
+      *Log(theErrLog) << "-d value should be an integer" << endl;
+      return false;
+    }
     Opts.Delete('d');
   };
   if ( Opts.Find ('n', value, mood)) {
@@ -187,6 +202,10 @@ bool parse_args( TimblOpts& Opts ) {
       doTok = false;
     if ( skip.find_first_of("mM") != string::npos )
       doMwu = false;
+    if ( skip.find_first_of("cC") != string::npos )
+      doIOB = false;
+    if ( skip.find_first_of("nN") != string::npos )
+      doNER = false;
     if ( skip.find_first_of("pP") != string::npos )
       doParse = false;
     Opts.Delete("skip");
@@ -196,10 +215,24 @@ bool parse_args( TimblOpts& Opts ) {
     encoding = value;
   }
   
+  if ( Opts.Find( "max-parser-tokens", value, mood ) ){
+    if ( value.empty() ){
+      *Log(theErrLog) << "max-parser-tokens option without value " << endl;
+      return false;
+    }
+    else {
+      if ( !Timbl::stringTo<unsigned int>( value, maxParserTokens ) ){
+	*Log(theErrLog) << "max-parser-tokens value should be an integer" << endl;
+	return false;
+      }
+    }
+    Opts.Delete("max-parser-tokens");
+  }
   if ( Opts.Find( "keep-parser-files", value, mood ) ){
     if ( value.empty() ||
 	 value == "true" || value == "TRUE" || value =="yes" || value == "YES" )
-      keepIntermediateFiles = true;
+      configuration.set( "keepIntermediateFiles", "true", "parser" );
+    Opts.Delete("keep-parser-files");
   }
   tmpDirName = configuration.lookUp( "tmpdir", "global" );
   if ( Opts.Find ( "tmpdir", value, mood)) {
@@ -274,7 +307,11 @@ bool parse_args( TimblOpts& Opts ) {
     outputFileName = value;
     Opts.Delete('o');
   };
-  wantXML = false;
+  doXMLout = false;
+  if ( Opts.Find ( "id", value, mood)) {
+    docid = value;
+    Opts.Delete( "id");
+  }  
   if ( Opts.Find( "xmldir", value, mood)) {
     xmlDirName = value;
 #ifdef HAVE_DIRENT_H
@@ -285,15 +322,35 @@ bool parse_args( TimblOpts& Opts ) {
       }
     }
 #endif
-    wantXML = true;
+    doXMLout = true;
     Opts.Delete( "xmldir");
   }
   else if ( Opts.Find ('X', value, mood)) {
-    wantXML = true;
+    doXMLout = true;
     XMLoutFileName = value;
     Opts.Delete('X');
   }
   
+  doXMLin = false;
+  if ( Opts.Find ('x', value, mood)) {
+    doXMLin = true;
+    if ( !value.empty() ){
+      if ( ! (xmlDirName.empty() && 
+	      testDirName.empty() && 
+	      TestFileName.empty() ) ){
+	*Log(theErrLog) << "-x may not provide a value when --testdir or --xmldir is provided" << endl;
+	return false;
+      }
+      TestFileName = value;
+      ifstream is( value.c_str() );
+      if ( !is ){
+	*Log(theErrLog) << "input stream " << value << " is not readable" << endl;
+	return false;
+      }
+    }
+    Opts.Delete('x');
+  }
+  
   if ( !outputFileName.empty() && !testDirName.empty() ){
     *Log(theErrLog) << "useless -o value" << endl;
     return false;
@@ -312,32 +369,6 @@ bool parse_args( TimblOpts& Opts ) {
     return false;
   }
 
-  string tagset = configuration.lookUp( "settings", "tagger" );
-  if ( tagset.empty() ){
-    *Log(theErrLog) << "Unable to find settings for Tagger" << endl;
-    return false;
-  }
-
-  string rulesName = configuration.lookUp( "rulesFile", "tokenizer" );
-  if ( rulesName.empty() ){
-    *Log(theErrLog) << "no rulesFile found in configuration" << endl;
-    return false;
-  }
-  else {
-    tokenizer.setErrorLog( theErrLog );
-    if ( !tokenizer.init( rulesName ) )
-      return false;
-  }
-  string debug = configuration.lookUp( "debug", "tokenizer" );
-  if ( debug.empty() )
-    tokenizer.setDebug( tpDebug );
-  else
-    tokenizer.setDebug( Timbl::stringTo<int>(debug) );
-
-  if ( Opts.Find ('Q', value, mood)) {
-      tokenizer.setQuoteDetection(true);
-  };
-
   if ( !doServer && TestFileName.empty() && fileNames.empty() ){
     *Log(theErrLog) << "no frogging without input!" << endl;
     return false;
@@ -345,26 +376,60 @@ bool parse_args( TimblOpts& Opts ) {
   return true;
 }
 
-bool init(){  
+bool froginit(){  
+  // for some modules init can take a long time
+  // so first make sure it will not fail on some trivialities
+  //
+  if ( doTok && !configuration.hasSection("tokenizer") ){
+    *Log(theErrLog) << "Missing [[tokenizer]] section in config file." << endl;
+    return false;
+  }
+  if ( doIOB && !configuration.hasSection("IOB") ){
+    *Log(theErrLog) << "Missing [[IOB]] section in config file." << endl;
+    return false;
+  }
+  if ( doNER && !configuration.hasSection("NER") ){
+    *Log(theErrLog) << "Missing [[NER]] section in config file." << endl;
+    return false;
+  }
+  if ( doMwu ){
+    if ( !configuration.hasSection("mwu") ){
+      *Log(theErrLog) << "Missing [[mwu]] section in config file." << endl;
+      return false;
+    }
+  }
+  else if ( doParse ){
+    *Log(theErrLog) << " Parser disabled, because MWU is deselected" << endl;
+    doParse = false;
+  }
+  
   if ( doServer ){
     // we use fork(). omp (GCC version) doesn't do well when omp is used
     // before the fork!
     // see: http://bisqwit.iki.fi/story/howto/openmp/#OpenmpAndFork
-    bool stat = myCGNTagger.init( configuration );
+    bool stat = tokenizer.init( configuration, docid, !doTok );
     if ( stat ){
-      bool stat = myMblem.init( configuration );
+      tokenizer.setSentencePerLineInput( doSentencePerLine );
+      tokenizer.setInputEncoding( encoding );
+      tokenizer.setInputXml( doXMLin );
+      stat = myCGNTagger.init( configuration );
       if ( stat ){
-	stat = myMbma.init( configuration );
-	if ( stat ) {
-	  if ( doMwu ){
-	    stat = myMwu.init( configuration );
-	    if ( stat && doParse )
-	      stat = myParser.init( configuration );
-	  }
-	  else {
-	    if ( doParse )
-	      *Log(theErrLog) << " Parser disabled, because MWU is deselected" << endl;
-	    doParse = false;;
+	if ( doIOB )
+	  stat = myIOBTagger.init( configuration );
+	if ( stat && doNER )
+	  stat = myNERTagger.init( configuration );
+	if ( stat ){
+	  stat = myMblem.init( configuration );
+	  if ( stat ){
+	    stat = myMbma.init( configuration );
+	    if ( stat ) {
+	      if ( doMwu ){
+		stat = myMwu.init( configuration );
+		if ( stat && doParse ){
+		  stat = myParser.init( configuration );
+		}
+	      }
+	    }
 	  }
 	}
       }
@@ -375,19 +440,47 @@ bool init(){
     }
   }
   else {
+    bool tokStat = true;
     bool lemStat = true;
     bool mwuStat = true;
     bool mbaStat = true;
     bool parStat = true;
     bool tagStat = true;
+    bool iobStat = true;
+    bool nerStat = true;
 #pragma omp parallel sections
     {
 #pragma omp section
-      lemStat = myMblem.init( configuration );
+      {
+	tokStat = tokenizer.init( configuration, docid, !doTok );
+	if ( tokStat ){
+	  tokenizer.setSentencePerLineInput( doSentencePerLine );
+	  tokenizer.setInputEncoding( encoding );
+	  tokenizer.setInputXml( doXMLin );
+	}
+      }
 #pragma omp section
-      mbaStat = myMbma.init( configuration );
+      {
+	lemStat = myMblem.init( configuration );
+      }
+#pragma omp section
+      {
+	mbaStat = myMbma.init( configuration );
+      }
 #pragma omp section 
-      tagStat = myCGNTagger.init( configuration );
+      {
+	tagStat = myCGNTagger.init( configuration );
+      }
+#pragma omp section 
+      {
+	if ( doIOB )
+	  iobStat = myIOBTagger.init( configuration );
+      }
+#pragma omp section 
+      {
+	if ( doNER )
+	  nerStat = myNERTagger.init( configuration );
+      }
 #pragma omp section
       {
 	if ( doMwu ){
@@ -400,18 +493,23 @@ bool init(){
 	    *Log(theErrLog) << "init Parse took: " << initTimer << endl;
 	  }
 	}
-	else {
-	  if ( doParse )
-	    *Log(theErrLog) << " Parser disabled, because MWU is deselected" << endl;
-	  doParse = false;
-	}
       }
     }   // end omp parallel sections
-    if ( ! ( tagStat && lemStat && mbaStat && mwuStat && parStat ) ){
-      *Log(theErrLog) << "Initialization failed: ";
+    if ( ! ( tokStat && iobStat && nerStat && tagStat && lemStat 
+	     && mbaStat && mwuStat && parStat ) ){
+      *Log(theErrLog) << "Initialization failed for: ";
+      if ( ! ( tokStat ) ){
+	*Log(theErrLog) << "[tokenizer] ";
+      }	
       if ( ! ( tagStat ) ){
 	*Log(theErrLog) << "[tagger] ";
       }	
+      if ( ! ( iobStat ) ){
+	*Log(theErrLog) << "[IOB] ";
+      }	
+      if ( ! ( nerStat ) ){
+	*Log(theErrLog) << "[NER] ";
+      }	
       if ( ! ( lemStat ) ){
 	*Log(theErrLog) << "[lemmatizer] ";
       }	
@@ -432,67 +530,12 @@ bool init(){
   return true;
 }
 
-void TestSentence( FoliaElement* sent,
-		   const string& tmpDir,
-		   TimerBlock& timers ){
-  vector<Word*> swords = sent->words();
-  if ( !swords.empty() ) {
-    timers.tagTimer.start();
-    string tagged = myCGNTagger.Classify( sent );
-    timers.tagTimer.stop();
-    for ( size_t i = 0; i < swords.size(); ++i ) {
-      string mbmaLemma;
-      string mblemLemma;
-#pragma omp parallel sections
-      {
-#pragma omp section
-	{
-	  timers.mbmaTimer.start();
-	  if (tpDebug) cout << "Calling mbma..." << endl;
-	  mbmaLemma = myMbma.Classify( swords[i] );
-	  timers.mbmaTimer.stop();
-	}
-#pragma omp section
-	{
-	  timers.mblemTimer.start();
-	  if (tpDebug) cout << "Calling mblem..." << endl;
-	  mblemLemma = myMblem.Classify( swords[i] );
-	  timers.mblemTimer.stop();
-	}
-      }
-      if (tpDebug) {
-	cout   << "tagged word[" << i <<"]: " << swords[i]->str() << " tag "
-	       << swords[i]->pos() << endl;
-	cout << "analysis: " << mblemLemma << " " << mbmaLemma << endl;
-      }
-    } //for int i = 0 to num_words
-
-    if ( doMwu ){
-      if ( swords.size() > 0 ){
-	if (tpDebug)
-	  cout << "starting mwu Chunking ... \n";
-	timers.mwuTimer.start();
-	myMwu.Classify( sent );
-	timers.mwuTimer.stop();
-      }
-      if (tpDebug) {
-	cout << "\n\nfinished mwu chunking!\n";
-	cout << myMwu << endl;
-      }
-    }
-    if ( doParse ){  
-      myParser.Parse( sent, tmpDir, timers );
-    }
-  }
-}
-
 vector<Word*> lookup( Word *word, const vector<Entity*>& entities ){
   vector<Word*> vec;
   for ( size_t p=0; p < entities.size(); ++p ){
     vec = entities[p]->select<Word>();
     if ( !vec.empty() ){
       if ( vec[0]->id() == word->id() ) {
-	// cerr << "found " << vec << endl;
 	return vec;
       }
     }
@@ -528,6 +571,101 @@ Dependency *lookupDep( const Word *word,
   return 0;
 }
 
+string lookupNEREntity( const vector<Word *>& mwu, 
+			const vector<Entity*>& entities ){
+  string endresult;
+  for ( size_t j=0; j < mwu.size(); ++j ){
+    if ( tpDebug ){
+      using folia::operator<<;
+      *Log(theErrLog) << "lookup "<< mwu[j] << " in " << entities << endl;
+    }
+    string result;
+    for ( size_t i=0; i < entities.size(); ++i ){
+      if ( tpDebug ){
+	*Log(theErrLog) << "probeer " << entities[i] << endl;
+      }
+      try {
+	vector<Word*> v = entities[i]->select<Word>();
+	bool first = true;
+	for ( size_t k=0; k < v.size(); ++k ){
+	  if ( v[k] == mwu[j] ){
+	    if (tpDebug){
+	      *Log(theErrLog) << "found word " << v[k] << endl;
+	    }
+	    if ( first )
+	      result += "B-" + uppercase(entities[i]->cls());
+	    else
+	      result += "I-" + uppercase(entities[i]->cls());
+	    break;
+	  }
+	  else
+	    first = false;
+	}
+      }
+      catch ( exception& e ){
+	if  (tpDebug > 0) 
+	  *Log(theErrLog) << "get NER results failed: " 
+			  << e.what() << endl;      
+      }
+    }
+    if ( result.empty() )
+      endresult += "O";
+    else
+      endresult += result;
+    if ( j < mwu.size()-1 )
+      endresult += "_";
+  }
+  return endresult;
+}
+
+
+string lookupIOBChunk( const vector<Word *>& mwu, 
+		       const vector<Chunk*>& chunks ){
+  string endresult;
+  for ( size_t j=0; j < mwu.size(); ++j ){
+    if ( tpDebug ){
+      using folia::operator<<;
+      *Log(theErrLog) << "lookup "<< mwu[j] << " in " << chunks << endl;
+    }
+    string result;
+    for ( size_t i=0; i < chunks.size(); ++i ){
+      if ( tpDebug ){
+	*Log(theErrLog) << "probeer " << chunks[i] << endl;
+      }
+      try {
+	vector<Word*> v = chunks[i]->select<Word>();
+	bool first = true;
+	for ( size_t k=0; k < v.size(); ++k ){
+	  if ( v[k] == mwu[j] ){
+	    if (tpDebug){
+	      *Log(theErrLog) << "found word " << v[k] << endl;
+	    }
+	    if ( first )
+	      result += "B-" + chunks[i]->cls();
+	    else
+	      result += "I-" + chunks[i]->cls();
+	    break;
+	  }
+	  else
+	    first = false;
+	}
+      }
+      catch ( exception& e ){
+	if  (tpDebug > 0) 
+	  *Log(theErrLog) << "get Chunks results failed: " 
+			  << e.what() << endl;      
+      }
+    }
+    if ( result.empty() )
+      endresult += "O";
+    else
+      endresult += result;
+    if ( j < mwu.size()-1 )
+      endresult += "_";
+  }
+  return endresult;
+}
+
 void displayMWU( ostream& os, size_t index, 
 		 const vector<Word*> mwu ){
   string wrd;
@@ -586,16 +724,23 @@ void displayMWU( ostream& os, size_t index,
   os << index << "\t" << wrd << "\t" << lemma << "\t" << morph << "\t" << pos << "\t" << std::fixed << conf;
 }  
 
-ostream &showResults( ostream& os, const FoliaElement* sentence ){
+ostream &showResults( ostream& os, 
+		      const Sentence* sentence,
+		      bool showParse ){
   vector<Word*> words = sentence->words();
-  vector<Entity*> entities = sentence->select<Entity>();
+  vector<Entity*> mwu_entities = sentence->select<Entity>( myMwu.getTagset() );
+  // using folia::operator<<;
+  // *Log(theErrLog) << "mwu entities " << mwu_entities << endl;
   vector<Dependency*> dependencies = sentence->select<Dependency>();
+  vector<Chunk*> iob_chunking = sentence->select<Chunk>();
+  vector<Entity*> ner_entities = sentence->select<Entity>( myNERTagger.getTagset() );
+  //  *Log(theErrLog) << "ner entities " << ner_entities << endl;
   size_t index = 1;
   map<FoliaElement*, int> enumeration;
   vector<vector<Word*> > mwus;
   for( size_t i=0; i < words.size(); ++i ){
     Word *word = words[i];
-    vector<Word*> mwu = lookup( word, entities );
+    vector<Word*> mwu = lookup( word, mwu_entities );
     for ( size_t j=0; j < mwu.size(); ++j ){
       enumeration[mwu[j]] = index;
     }
@@ -605,7 +750,23 @@ ostream &showResults( ostream& os, const FoliaElement* sentence ){
   }
   for( size_t i=0; i < mwus.size(); ++i ){
     displayMWU( os, i+1, mwus[i] );
-    if ( doParse ){
+    if ( doNER ){
+      string cls;
+      string s = lookupNEREntity( mwus[i], ner_entities );
+      os << "\t" << s;
+    }
+    else {
+      os << "\t\t";
+    }
+    if ( doIOB ){
+      string cls;
+      string s = lookupIOBChunk( mwus[i], iob_chunking );
+      os << "\t" << s;
+    }
+    else {
+      os << "\t\t";
+    }
+    if ( showParse ){
       string cls;
       Dependency *dep = lookupDep( mwus[i][0], dependencies );
       if ( dep ){
@@ -627,53 +788,142 @@ ostream &showResults( ostream& os, const FoliaElement* sentence ){
   return os;
 }
 
+void TestSentence( const vector<Sentence*>& sentences,
+		   size_t index,
+		   ostream& outStream,
+		   TimerBlock& timers ){
+  Sentence *sent = sentences[index];
+  vector<Word*> swords = sent->words();
+  bool showParse = doParse;
+  if ( !swords.empty() ) {
+#pragma omp parallel sections
+    {
+#pragma omp section
+      {
+	timers.tagTimer.start();
+	myCGNTagger.Classify( sent );
+	timers.tagTimer.stop();
+      }
+#pragma omp section
+      {
+	if ( doIOB ){
+	  timers.iobTimer.start();
+	  myIOBTagger.Classify( sent );
+	  timers.iobTimer.stop();
+	}
+      }
+#pragma omp section
+      {
+	if ( doNER ){
+	  timers.nerTimer.start();
+	  myNERTagger.Classify( sent );
+	  timers.nerTimer.stop();
+	}
+      }
+    } // parallel sections
+    for ( size_t i = 0; i < swords.size(); ++i ) {
+      string mbmaLemma;
+      string mblemLemma;
+#pragma omp parallel sections
+      {
+#pragma omp section
+	{
+	  timers.mbmaTimer.start();
+	  if (tpDebug) 
+	    *Log(theErrLog) << "Calling mbma..." << endl;
+	  mbmaLemma = myMbma.Classify( swords[i] );
+	  timers.mbmaTimer.stop();
+	}
+#pragma omp section
+	{
+	  timers.mblemTimer.start();
+	  if (tpDebug) 
+	    *Log(theErrLog) << "Calling mblem..." << endl;
+	  mblemLemma = myMblem.Classify( swords[i] );
+	  timers.mblemTimer.stop();
+	}
+      } // omp parallel sections
+      if (tpDebug) {
+	*Log(theErrLog) << "tagged word[" << i <<"]: " << swords[i]->str() 
+			<< " tag " << swords[i]->pos() << endl;
+	*Log(theErrLog) << "analysis: " << mblemLemma << " " << mbmaLemma << endl;
+      }
+    } //for int i = 0 to num_words
+    
+    if ( doMwu ){
+      if ( swords.size() > 0 ){
+	timers.mwuTimer.start();
+	myMwu.Classify( sent );
+	timers.mwuTimer.stop();
+      }
+    }
+    if ( doParse ){
+      if ( maxParserTokens != 0 && swords.size() > maxParserTokens ){
+	*Log(theErrLog) << "WARNING!" << endl;
+	*Log(theErrLog) << "Sentence " << index+1 << " isn't parsed because it contains " << swords.size() << " tokens, and you provided the --max-parser-tokens=" << maxParserTokens << " option." << endl;
+	showParse = false;
+      }
+      else {
+	myParser.Parse( sent, myMwu.getTagset(), tmpDirName, timers );
+      }
+    }
+  }
+  if ( !(doServer && doXMLout) )
+    showResults( outStream, sent, showParse );
+}
 
-
-void Test( istream& IN,
+void Test( Document& doc,
 	   ostream& outStream,
-	   TimerBlock& timers,
-	   Common::Timer &frogTimer,
-	   const string& xmlOutFile,
-	   const string& tmpDir ) {
-
-  string eosmarker = "";
-  tokenizer.setEosMarker( eosmarker );
-  tokenizer.setVerbose( false );
-  tokenizer.setSentenceDetection( true ); //detection of sentences
-  tokenizer.setParagraphDetection( false ); //detection of paragraphs
-  tokenizer.setSentencePerLineInput( doSentencePerLine );
-  if ( !encoding.empty() )
-    tokenizer.setInputEncoding( encoding );
-  
-  tokenizer.setXMLOutput( true, "frog" );
-  string line;  
-  Document doc = tokenizer.tokenize( IN ); 
-  doc.declare( AnnotationType::POS, "mbt-pos");
-  doc.declare( AnnotationType::LEMMA, "mbt-lemma");
-
-  // Tokenize the whole input into one FoLiA document.
-  // This is nog a good idea on the long term, I think
-
+	   const string& xmlOutFile = "" ) {
+  TimerBlock timers;
+  timers.frogTimer.start();
+  // first we make sure that the doc will accept our annotations, by
+  // declaring them in the doc
+  myCGNTagger.addDeclaration( doc );
+  myMblem.addDeclaration( doc );
+  myMbma.addDeclaration( doc );
+  if (doIOB)
+    myIOBTagger.addDeclaration( doc );
+  if (doNER)
+    myNERTagger.addDeclaration( doc );
+  if (doMwu) 
+    myMwu.addDeclaration( doc );
+  if (doParse) 
+    myParser.addDeclaration( doc );
+  if ( tpDebug > 5 )
+    *Log(theErrLog) << "Testing document :" << doc << endl;
   vector<Sentence*> sentences = doc.sentences();
   size_t numS = sentences.size();
   if ( numS > 0 ) { //process sentences 
-    if  (tpDebug > 0) *Log(theErrLog) << "[tokenize] " << numS << " sentence(s) in buffer, processing..." << endl;
+    if  (tpDebug > 0) 
+      *Log(theErrLog) << "found " << numS << " sentence(s) in document." << endl;
     for ( size_t i = 0; i < numS; i++) {
       /* ******* Begin process sentence  ********** */
-      TestSentence( sentences[i],  tmpDir, timers ); 
-      //NOTE- full sentences are passed (which may span multiple lines) (MvG)         
-      showResults( outStream, sentences[i] ); 
+      TestSentence( sentences, i, outStream, timers ); 
+      //NOTE- full sentences are passed (which may span multiple lines) (MvG)
     }
-  } else {
-    if  (tpDebug > 0) *Log(theErrLog) << "[tokenize] No sentences yet, reading on..." << endl;
+    if ( doServer && doXMLout )
+      outStream << doc << endl;
+    if ( !xmlOutFile.empty() ){
+      doc.save( xmlOutFile );
+      *Log(theErrLog) << "resulting FoLiA doc saved in " << xmlOutFile << endl;
+    }
+  } 
+  else {
+    if  (tpDebug > 0) 
+      *Log(theErrLog) << "No sentences found in document. " << endl;
   }
   
-  frogTimer.stop();  
+  timers.frogTimer.stop();  
   
-  *Log(theErrLog) << "tokenisation took:" << timers.tokTimer << endl;
-  *Log(theErrLog) << "tagging took:     " << timers.tagTimer << endl;
-  *Log(theErrLog) << "MBA took:         " << timers.mbmaTimer << endl;
-  *Log(theErrLog) << "Mblem took:       " << timers.mblemTimer << endl;
+  *Log(theErrLog) << "tokenisation took:  " << timers.tokTimer << endl;
+  *Log(theErrLog) << "CGN tagging took:   " << timers.tagTimer << endl;
+  if ( doIOB)
+    *Log(theErrLog) << "IOB chunking took:  " << timers.iobTimer << endl;
+  if ( doNER)
+    *Log(theErrLog) << "NER took:           " << timers.nerTimer << endl;
+  *Log(theErrLog) << "MBA took:           " << timers.mbmaTimer << endl;
+  *Log(theErrLog) << "Mblem took:         " << timers.mblemTimer << endl;
   if ( doMwu )
     *Log(theErrLog) << "MWU resolving took: " << timers.mwuTimer << endl;
   if ( doParse ){
@@ -684,81 +934,94 @@ void Test( istream& IN,
     *Log(theErrLog) << "Parsing (csi)     took: " << timers.csiTimer << endl;
     *Log(theErrLog) << "Parsing (total)   took: " << timers.parseTimer << endl;
   }
-  *Log(theErrLog) << "Frogging took:      " << frogTimer << endl;
-  if ( !xmlOutFile.empty() ){
-    doc.save( xmlOutFile );
-    *Log(theErrLog) << "resulting FoLiA doc saved in " << xmlOutFile << endl;
-  }
+  *Log(theErrLog) << "Frogging in total took: " << timers.frogTimer << endl;
   return;
 }
 
-void TestFile( const string& infilename, 
-	       const string& outFileName,
-	       const string& xmlOutFile ) {
-  // init's are done
-  TimerBlock timers;
-  Common::Timer frogTimer;
-  frogTimer.start();
-
-  string TestFileName = ""; //untokenised!
-  
-  ofstream outStream;
-  if ( !outFileName.empty() ){
-    if ( outStream.open( outFileName.c_str() ), outStream.bad() ){
-      *Log(theErrLog) << "unable to open outputfile: " << outFileName << endl;
+void Test( const string& infilename,
+	   const string& outName,
+	   const string& xmlOutFile ) {
+  // stuff the whole input into one FoLiA document.
+  // This is not a good idea on the long term, I think (agreed [proycon] )
+  ostream *os;
+  if ( outName.empty() ){
+    os = &cout;
+  }
+  else {
+    os = new ofstream( outName.c_str() );
+    if ( os->bad() ){
+      *Log(theErrLog) << "unable to open outputfile: " << outName << endl;
       exit( EXIT_FAILURE );
     }
   }
-  TestFileName = infilename; //untokenised!
-
-  ifstream IN( TestFileName.c_str() );
-
-  if (!outFileName.empty()) {
-    Test(IN, outStream, timers, frogTimer, xmlOutFile, tmpDirName );
-  } else {
-    Test(IN, cout, timers, frogTimer, xmlOutFile, tmpDirName );
+  if ( doXMLin ){
+    Document doc;
+    doc.readFromFile( infilename );
+    tokenizer.tokenize( doc );
+    Test( doc, *os, xmlOutFile );
+  }
+  else {
+    ifstream IN( infilename.c_str() );
+    Document doc = tokenizer.tokenize( IN );
+    Test( doc, *os, xmlOutFile );
+  }
+  if ( !outName.empty() ){
+    *Log(theErrLog) << "results stored in " << outName << endl;
+    delete os;
   }
-  
-  if ( !outFileName.empty() )
-    *Log(theErrLog) << "results stored in " << outFileName << endl;
 }
 
-
 void TestServer( Sockets::ServerSocket &conn) {
   //by Maarten van Gompel
-
   
-  
-
   try {
     while (true) {
-      string data = "";      
-      while (true) {
-	string tmpdata;
-	if ( !conn.read( tmpdata ) )	 //read data from client
-	    throw( runtime_error( "read failed" ) );
-	
-	if (data.length() < 2048) { /* Todo: get TCP_BUFFER_SIZE dynamically from TimblServer and OS */
-	    data = data + tmpdata;
+      ostringstream outputstream;
+      if ( doXMLin ){
+	string result;
+	string s;
+	while ( conn.read(s) ){
+	  result += s + "\n";
+	  if ( s.empty() )
 	    break;
 	}
+	if ( result.size() < 50 ){
+	  // a FoLia doc must be at least a few 100 bytes
+	  // so this is wrong. Just bail out
+	  throw( runtime_error( "read garbage" ) );
+	}
+	if ( tpDebug )
+	  *Log(theErrLog) << "received data [" << result << "]" << endl;	
+	Document doc;
+	try {
+	  doc.readFromString( result );
+	}
+	catch ( std::exception& e ){
+	  *Log(theErrLog) << "FoLiaParsing failed:" << endl
+			  << e.what() << endl;	  
+	  throw;
+	}
+	*Log(theErrLog) << "Processing... " << endl;
+	tokenizer.tokenize( doc );
+	Test( doc, outputstream );
+      }
+      else {
+	string data = "";      
+	if ( !conn.read( data ) )	 //read data from client
+	  throw( runtime_error( "read failed" ) );
+	if (tpDebug)
+	  *Log(theErrLog) << "Received: [" << data << "]" << endl;
+	*Log(theErrLog) << "Processing... " << endl;
+	istringstream inputstream(data,istringstream::in);
+	Document doc = tokenizer.tokenize( inputstream ); 
+	Test( doc, outputstream );
+      }
+      if (!conn.write( (outputstream.str()) ) || !(conn.write("READY\n"))  ){
+	if (tpDebug)
+	  *Log(theErrLog) << "socket " << conn.getMessage() << endl;	
+	throw( runtime_error( "write to client failed" ) );
       }
-          	
-      if (tpDebug)
-	std::cerr << "Received: [" << data << "]" << "\n";
       
-      istringstream inputstream(data,istringstream::in);
-      ostringstream outputstream;
-
-      *Log(theErrLog) << "Processing... " << endl;
-      
-      TimerBlock timers;
-  	  Common::Timer frogTimer;
-  	  frogTimer.start();
-      Test(inputstream, outputstream, timers, frogTimer, "", tmpDirName );
-      if (!conn.write( (outputstream.str()) ) || !(conn.write("READY\n"))  )
-	  throw( runtime_error( "write to client failed" ) );
-
     }
   }
   catch ( std::exception& e ) {
@@ -779,13 +1042,18 @@ int main(int argc, char *argv[]) {
        << Timbl::VersionName() << ", "
        << TimblServer::VersionName() << ", "
        << Tagger::VersionName() << "]" << endl;
+  cout << "configdir: " << configDir << endl;
   try {
     TimblOpts Opts(argc, argv);
         
-    if ( parse_args(Opts) &&
-	 init() ){
-      if ((tpDebug) || (doServer)) {
-      	//don't mangle debug output, so run 1 thread then.. also run in one thread in server mode, forking is too expensive for lots of small snippets
+    if ( parse_args(Opts) ){
+      if (  !froginit() ){
+	cerr << "terminated." << endl;
+	return EXIT_FAILURE;
+      }
+      
+      if ( doServer ) {
+      	// run in one thread in server mode, forking is too expensive for lots of small snippets
       	omp_set_num_threads( 1 );
       }
       if ( !fileNames.empty() ) {
@@ -798,13 +1066,23 @@ int main(int argc, char *argv[]) {
 	set<string>::const_iterator it = fileNames.begin();
 	while ( it != fileNames.end() ){
 	  string testName = testDirName +"/" + *it;
-	  string outName = outPath + *it + ".out";
+	  string outName;
+	  if ( doXMLin ){
+	    if ( !outPath.empty() )
+	      outName = outPath + *it + ".out";
+	  }
+	  else
+	    outName = outPath + *it + ".out";
 	  string xmlName;
-	  if ( !xmlDirName.empty() )
-	    xmlName = xmlPath + *it + ".xml";
-	  else if ( wantXML )
+	  if ( !xmlDirName.empty() ){
+	    if ( it->rfind(".xml") == string::npos )
+	      xmlName = xmlPath + *it + ".xml";
+	    else
+	      xmlName = xmlPath + *it;
+	  }
+	  else if ( doXMLout )
 	    xmlName = *it + ".xml"; // do not clobber the inputdir!
-	  TestFile( testName, outName, xmlName );
+	  Test( testName, outName, xmlName );
 	  ++it;
 	}
       }
@@ -820,7 +1098,7 @@ int main(int argc, char *argv[]) {
 	
 	srand((unsigned)time(0));
 	
-	std::cerr << "Listening on port " << listenport << "\n";
+	*Log(theErrLog) << "Listening on port " << listenport << "\n";
 	
 	try
 	  {
@@ -836,10 +1114,10 @@ int main(int argc, char *argv[]) {
 	      
 	      Sockets::ServerSocket conn;
 	      if ( server.accept( conn ) ){
-		std::cerr << "New connection...\n";
+		*Log(theErrLog) << "New connection..." << endl;
 		int pid = fork();				
 		if (pid < 0) {
-		  std::cerr << "ERROR on fork\n";
+		  *Log(theErrLog) << "ERROR on fork" << endl;
 		  exit(EXIT_FAILURE);
 		} else if (pid == 0)  {
 		  //		  server = NULL;
@@ -853,16 +1131,16 @@ int main(int argc, char *argv[]) {
 	    }
 	  } catch ( std::exception& e )
 	  {
-	    std::cerr << "Server error:" << e.what() << "\nExiting.\n";
+	    *Log(theErrLog) << "Server error:" << e.what() << " Exiting." << endl;
 	    exit(EXIT_FAILURE);
 	  }
 	
       } else {
-	if ( wantXML && XMLoutFileName.empty() )
+	if ( doXMLout && XMLoutFileName.empty() )
 	  XMLoutFileName = TestFileName + ".xml";
 	if ( wantOUT && outputFileName.empty() )
 	  outputFileName = TestFileName + ".out";
-	TestFile( TestFileName, outputFileName, XMLoutFileName );
+	Test( TestFileName, outputFileName, XMLoutFileName );
       }
     }
     else {
