@@ -1,6 +1,6 @@
 /*
-  $Id: Frog.cxx 13560 2011-11-22 13:51:45Z sloot $
-  $URL: https://ilk.uvt.nl/svn/sources/Frog/trunk/src/Frog.cxx $
+  $Id: cgn_tagger_mod.cxx 14489 2012-03-22 11:37:02Z sloot $
+  $URL: https://ilk.uvt.nl/svn/sources/Frog/trunk/src/cgn_tagger_mod.cxx $
 
   Copyright (c) 2006 - 2012
   Tilburg University
@@ -30,18 +30,21 @@
 
 #include "mbt/MbtAPI.h"
 #include "libfolia/folia.h"
+#include "libfolia/document.h"
 #include "frog/Frog.h"
 #include "frog/cgn_tagger_mod.h"
 
 using namespace std;
 using namespace folia;
 
-MBTagger::MBTagger(){
+CGNTagger::CGNTagger(){
   tagger = 0;
+  cgnLog = new LogStream( theErrLog, "cgn-tagger-" );
 }
 
-MBTagger::~MBTagger(){
+CGNTagger::~CGNTagger(){
   delete tagger;
+  delete cgnLog;
 }
 
 static multimap<string,string> cgnSubSets;
@@ -173,107 +176,56 @@ void fillSubSetTable(){
 }
 
  
-bool MBTagger::init( const Configuration& conf ){
+bool CGNTagger::init( const Configuration& conf ){
+  debug = tpDebug;
+  string db = conf.lookUp( "debug", "tagger" );
+  if ( !db.empty() )
+    debug = folia::stringTo<int>( db );
+  switch ( debug ){
+  case 0:
+  case 1:
+    break;
+  case 2:
+  case 3:
+  case 4:
+    cgnLog->setlevel(LogDebug);
+    break;
+  case 5:
+  case 6:
+  case 7:
+    cgnLog->setlevel(LogHeavy);
+    break;
+  default:
+    cgnLog->setlevel(LogExtreme);
+  }    
   if ( tagger != 0 ){
-    *Log(theErrLog) << "MBTagger is already initialized!" << endl;
+    *Log(cgnLog) << "CGNTagger is already initialized!" << endl;
     return false;
   }  
-  string tagset = conf.lookUp( "settings", "tagger" );
-  if ( tagset.empty() ){
-    *Log(theErrLog) << "Unable to find settings for Tagger" << endl;
+  string val = conf.lookUp( "settings", "tagger" );
+  if ( val.empty() ){
+    *Log(cgnLog) << "Unable to find settings for Tagger" << endl;
     return false;
   }
-  fillSubSetTable();
-  string init = "-s " + configuration.configDir() + tagset;
-  if ( Tagger::Version() == "3.2.6" )
-    init += " -vc";
+  string settings =  val;
+  val = conf.lookUp( "version", "tagger" );
+  if ( val.empty() ){
+    version = "1.0";
+  }
   else
-    init += " -vcf";
-  tagger = new MbtAPI( init, *theErrLog );
+    version = val;
+  val = conf.lookUp( "set", "tagger" );
+  if ( val.empty() ){
+    tagset = "http://ilk.uvt.nl/folia/sets/frog-mbpos-cgn";
+  }
+  else
+    tagset = val;
+  fillSubSetTable();
+  string init = "-s " + configuration.configDir() + settings + " -vcf";
+  tagger = new MbtAPI( init, *cgnLog );
   return tagger != 0;
 }
 
-static bool splitOneWT( const string& inp, string& word, string& tag, string& confidence ){
-  bool isKnown = true;
-  string in = inp;
-  //     split word and tag, and store num of slashes
-  if (tpDebug)
-    cout << "split Classify starting with " << in << endl;
-  string::size_type pos = in.rfind("/");
-  if ( pos == string::npos ) {
-    *Log(theErrLog) << "no word/tag/confidence triple in this line: " << in << endl;
-    exit( EXIT_FAILURE );
-  }
-  else {
-    confidence = in.substr( pos+1 );
-    in.erase( pos );
-  }
-  pos = in.rfind("//");
-  if ( pos != string::npos ) {
-    // double slash: lets's hope is is an unknown word
-    if ( pos == 0 ){
-      // but this is definitely something like //LET() 
-      word = "/";
-      tag = in.substr(pos+2);
-    }
-    else {
-      word = in.substr( 0, pos );
-      tag = in.substr( pos+2 );
-    }
-    isKnown = false;
-  } 
-  else {
-    pos = in.rfind("/");
-    if ( pos != string::npos ) {
-      word = in.substr( 0, pos );
-      tag = in.substr( pos+1 );
-    }
-    else {
-      *Log(theErrLog) << "no word/tag/confidence triple in this line: " << in << endl;
-      exit( EXIT_FAILURE );
-    }
-  }
-  if ( tpDebug){
-    if ( isKnown )
-      cout << "known";
-    else
-      cout << "unknown";
-    cout << " word: " << word << "\ttag: " << tag << "\tconfidence: " << confidence << endl;
-  }
-  return isKnown;
-}
-
-static int splitWT( const string& tagged,
-	     vector<string>& tags, vector<double>& conf ){
-  vector<string> words;
-  vector<string> tagwords;
-  vector<bool> known;
-  tags.clear();
-  conf.clear();
-  size_t num_words = Timbl::split_at( tagged, tagwords, " " );
-  num_words--; // the last "word" is <utt> which gets added by the tagger
-  for( size_t i = 0; i < num_words; ++i ) {
-    string word, tag, confs;
-    bool isKnown = splitOneWT( tagwords[i], word, tag, confs );
-    double confidence;
-    if ( !stringTo<double>( confs, confidence ) ){
-      *Log(theErrLog) << "tagger confused. Expected a double, got '" << confs << "'" << endl;
-      exit( EXIT_FAILURE );
-    }
-    words.push_back( word );
-    tags.push_back( tag );
-    known.push_back( isKnown );
-    conf.push_back( confidence );
-  }
-  if (tpDebug) {
-    cout << "#tagged_words: " << num_words << endl;
-    for( size_t i = 0; i < num_words; i++) 
-      cout   << "\ttagged word[" << i <<"]: " << words[i] << (known[i]?"/":"//")
-	     << tags[i] << " <" << conf[i] << ">" << endl;
-  }
-  return num_words;
-}
-
 string getSubSet( const string& val, const string& head ){
   typedef multimap<string,string>::const_iterator mm_iter;
   mm_iter it = cgnSubSets.find( val );
@@ -302,23 +254,22 @@ string getSubSet( const string& val, const string& head ){
 			   "' whithin the constraints for '" + head + "'" );
 }
 
-void addTag( FoliaElement *word, const string& inputTag, double confidence ){
+void CGNTagger::addTag( Word *word, const string& inputTag, double confidence ){
   string cgnTag = inputTag;
   string mainTag;
   string tagPartS;
-  string annotator = "MBT";
   string ucto_class = word->cls();
+  if ( debug )
+    *Log(cgnLog) << "ucto class= " << ucto_class << endl;
   if ( ucto_class == "ABBREVIATION-KNOWN" ){
-    annotator = "ucto";
     mainTag = "SPEC";
     tagPartS = "afk";
     cgnTag = "SPEC(afk)";
     confidence = 1.0;
   }
   else if ( ucto_class == "SMILEY" ||
-	    ucto_class == "URL-WWW" ||
+	    ucto_class.find("URL") == 0 ||
 	    ucto_class == "E-MAIL" ){
-    annotator = "ucto";
     mainTag = "SPEC";
     tagPartS = "symb";
     cgnTag = "SPEC(symb)";
@@ -336,24 +287,43 @@ void addTag( FoliaElement *word, const string& inputTag, double confidence ){
       confidence = 1.0;
     tagPartS = cgnTag.substr( openH+1, closeH-openH-1 );
   }
-  KWargs args = getArgs( "set='mbt-pos', head='" + mainTag
-			 + "', cls='" + cgnTag
-			 + "', annotator='" + annotator + "', confidence='" 
-			 + toString(confidence) + "'" );
-  FoliaElement *pos = word->addPosAnnotation( args );
+  KWargs args;
+  args["set"]  = tagset;
+  args["head"] = mainTag;
+  args["cls"]  = cgnTag;
+  args["confidence"]= toString(confidence);
+  folia::FoliaElement *pos = 0;
+#pragma omp critical(foliaupdate)
+  {
+    pos = word->addPosAnnotation( args );
+  }
   vector<string> tagParts;
   size_t numParts = Timbl::split_at( tagPartS, tagParts, "," );
   for ( size_t i=0; i < numParts; ++i ){
-    string arg = "set='mbt-pos', subset='" + getSubSet( tagParts[i], mainTag ) 
-      + "', cls='" + tagParts[i] + "', annotator='MBT'";
-    FoliaElement *feat = new folia::Feature( arg );
-    pos->append( feat );
+    KWargs args;
+    args["set"]    = tagset;
+    args["subset"] = getSubSet( tagParts[i], mainTag );
+    args["cls"]    = tagParts[i];
+#pragma omp critical(foliaupdate)
+    {
+      folia::Feature *feat = new folia::Feature( args );
+      pos->append( feat );
+    }
   }
 }
 
-string MBTagger::Classify( FoliaElement *sent ){
-  string tagged;
-  vector<Word*> swords = sent->words();
+void CGNTagger::addDeclaration( Document& doc ) const {
+  doc.declare( AnnotationType::POS, tagset,
+	       "annotator='frog-mbpos-" + version
+	       + "', annotatortype='auto'");
+}
+
+void CGNTagger::Classify( Sentence *sent ){
+  vector<Word*> swords;
+#pragma omp critical(foliaupdate)
+  {  
+    swords = sent->words();
+  }
   if ( !swords.empty() ) {
     vector<string> words;
     string sentence; // the tagger needs the whole sentence
@@ -363,24 +333,22 @@ string MBTagger::Classify( FoliaElement *sent ){
       if ( w < swords.size()-1 )
 	sentence += " ";
     }
-    if (tpDebug > 0 ) *Log(theErrLog) << "[DEBUG] SENTENCE: " << sentence << endl;
-    
-    if (tpDebug) 
-      cout << "in: " << sentence << endl;
-    tagged = tagger->Tag(sentence);
-    if (tpDebug) {
-      cout << "sentence: " << sentence
-	   <<"\ntagged: "<< tagged
-	   << endl;
+    if (debug) 
+      *Log(cgnLog) << "CGN tagger in: " << sentence << endl;
+    vector<TagResult> tagv = tagger->TagLine(sentence);
+    if ( tagv.size() != swords.size() ){
+      throw runtime_error( "CGN tagger is confused" );
     }
-    vector<double> conf;
-    vector<string> tags;
-    int num_words = splitWT( tagged, tags, conf );
-    for ( int i = 0; i < num_words; ++i ) {
-      addTag( swords[i], tags[i], conf[i] );
+    if ( debug ){
+      *Log(cgnLog) << "CGN tagger out: " << endl;
+      for ( size_t i=0; i < tagv.size(); ++i ){
+	*Log(cgnLog) << "[" << i << "] : word=" << tagv[i].word() 
+		     << " tag=" << tagv[i].assignedTag() 
+		     << " confidence=" << tagv[i].confidence() << endl;
+      }
+    }
+    for ( size_t i=0; i < tagv.size(); ++i ){
+      addTag( swords[i], tagv[i].assignedTag(), tagv[i].confidence() );
     }
   }
-  return tagged;
 }
-
-
