@@ -1,5 +1,5 @@
 /*
-  $Id: mbma_mod.cxx 13948 2012-01-11 10:44:24Z sloot $
+  $Id: mbma_mod.cxx 14501 2012-03-22 13:00:16Z sloot $
   $URL: https://ilk.uvt.nl/svn/sources/Frog/trunk/src/mbma_mod.cxx $
 
   Copyright (c) 2006 - 2012
@@ -36,6 +36,7 @@
 #include "frog/Frog.h"
 #include "frog/Configuration.h"
 #include "libfolia/folia.h"
+#include "libfolia/document.h"
 #include "frog/mbma_mod.h"
 
 using namespace std;
@@ -44,7 +45,14 @@ using namespace folia;
 const long int LEFT =  6; // left context
 const long int RIGHT = 6; // right context
 
-Mbma::Mbma(): MTreeFilename( "dm.igtree" ), MTree(0) { }
+Mbma::Mbma(): MTreeFilename( "dm.igtree" ), MTree(0) { 
+  mbmaLog = new LogStream( theErrLog, "mbma-" );
+}
+
+Mbma::~Mbma() { 
+  cleanUp(); 
+  delete mbmaLog;
+};
 
 void Mbma::fillMaps() {
   //
@@ -100,7 +108,7 @@ void Mbma::init_cgn( const string& dir ) {
       vector<string> tmp;
       size_t num = split_at(line, tmp, " ");
       if ( num < 2 ){
-	*Log(theErrLog) << "splitting '" << line << "' failed" << endl;
+	*Log(mbmaLog) << "splitting '" << line << "' failed" << endl;
 	throw ( runtime_error("panic") );
       }
       TAGconv.insert( make_pair( tmp[0], tmp[1] ) );
@@ -123,11 +131,23 @@ void Mbma::init_cgn( const string& dir ) {
 }
   
 bool Mbma::init( const Configuration& config ) {
-  *Log(theErrLog) << "Initiating morphological analyzer...\n";
+  *Log(mbmaLog) << "Initiating morphological analyzer..." << endl;
   debugFlag = tpDebug;
   string db = config.lookUp( "debug", "mbma" );
   if ( !db.empty() )
     debugFlag = stringTo<int>( db );
+  string val = config.lookUp( "version", "mbma" );
+  if ( val.empty() ){
+    version = "1.0";
+  }
+  else
+    version = val;
+  val = config.lookUp( "set", "mbma" );
+  if ( val.empty() ){
+    tagset = "http://ilk.uvt.nl/folia/sets/frog-mbma-nl";
+  }
+  else
+    tagset = val;
   string tfName = config.lookUp( "treeFile", "mbma" );
   if ( tfName.empty() )
     tfName = "mbma.igtree";
@@ -144,7 +164,7 @@ bool Mbma::init( const Configuration& config ) {
 }
   
 void Mbma::cleanUp(){
-  // *Log(theErrLog) << "cleaning up MBMA stuff " << endl;
+  // *Log(mbmaLog) << "cleaning up MBMA stuff " << endl;
   delete MTree;
   MTree = 0;
 }
@@ -152,29 +172,29 @@ void Mbma::cleanUp(){
 vector<string> Mbma::make_instances( const UnicodeString& word ){
   vector<string> insts;
   if (debugFlag > 2)
-    cout << "word: " << word << "\twl : " << word.length() << endl;
+    *Log(mbmaLog) << "word: " << word << "\twl : " << word.length() << endl;
   for( long i=0; i < word.length(); ++i ) {
     if (debugFlag > 10)
-      cout << "itt #:" << i << endl;
+      *Log(mbmaLog) << "itt #:" << i << endl;
     UnicodeString inst;
     for ( long j=i ; j <= i + RIGHT + LEFT; ++j ) {
       if (debugFlag > 10)
-	cout << " " << j-LEFT << ": ";
+	*Log(mbmaLog) << " " << j-LEFT << ": ";
       if ( j < LEFT || j >= word.length()+LEFT )
-	inst += "_,";
+	inst += '_';
       else {
 	if (word[j-LEFT] == ',' )
-	  inst += "C";
+	  inst += 'C';
 	else
 	  inst += word[j-LEFT];
-	inst += ",";
       }
+      inst += ",";
       if (debugFlag > 10)
-	cout << " : " << inst << endl;
+	*Log(mbmaLog) << " : " << inst << endl;
     }
     inst += "?";
     if (debugFlag > 2)
-      cout << "inst #" << i << " : " << inst << endl;
+      *Log(mbmaLog) << "inst #" << i << " : " << inst << endl;
     // classify res
     insts.push_back( UnicodeToUTF8(inst) );
     // store res
@@ -182,14 +202,6 @@ vector<string> Mbma::make_instances( const UnicodeString& word ){
   return insts;
 }
   
-string extract_from( const string& line, char from ){
-  size_t pos = line.find( from );
-  if ( pos != string::npos )
-    return line.substr( pos+1 );
-  else
-    return "";
-}
-
 string extract( const string& line, size_t start, char to ){
   return line.substr( start, line.find( to , start ) - start );
 }
@@ -216,7 +228,7 @@ string Mbma::calculate_ins_del( const string& in_class,
   size_t pos = in_class.find("+");
   if ( pos != string::npos ) { 
     if ( debugFlag){
-      cout << "calculate ins/del for " << in_class << endl;
+      *Log(mbmaLog) << "calculate ins/del for " << in_class << endl;
     }
     pos++;
     if (in_class[pos]=='D') { // delete operation 
@@ -230,14 +242,19 @@ string Mbma::calculate_ins_del( const string& in_class,
       pos += deletestring.length()+1;
       insertstring = extract( in_class, pos+1, '/' );
     }
-    /* spelling change done; remove from this_class */
+    /* spelling change done; remove from in_class */
     result_class = extract( in_class, 0, '+' );
     pos = in_class.find("/");
     if ( pos != string::npos )
       result_class += in_class.substr( pos );
     if ( debugFlag){
-      cout << "Insert = " << insertstring << ", delete=" << deletestring << endl;
-      cout << "result class=" << result_class << endl;
+      *Log(mbmaLog) << "Insert = " << insertstring << ", delete=" << deletestring << endl;
+      *Log(mbmaLog) << "result class=" << result_class << endl;
+    }
+  }
+  else {
+    if ( debugFlag ){
+      *Log(mbmaLog) << "calculate_ins_del: do nothing" << endl;
     }
   }
   return result_class;
@@ -264,10 +281,10 @@ vector<waStruct> Mbma::Step1( unsigned int step,
   string this_class;
   string previoustag;
   waStruct waItem;
-  for ( long k=0; k< word.length(); ++k ) { 
+  for ( long k=0; k < word.length(); ++k ) { 
     this_class = find_class( step, classParts[k], nranal );
     if ( debugFlag){
-      cout << "Step1::" << step << " " << this_class << endl;
+      *Log(mbmaLog) << "Step::" << step << " " << this_class << endl;
     }
     string deletestring;
     string insertstring;
@@ -289,9 +306,9 @@ vector<waStruct> Mbma::Step1( unsigned int step,
 	( insertstring != "be" ) )
       tobeignored = insertstring.length();
     
-    /* encountering POS tag */
     if ( basictags.find(this_class[0]) != string::npos &&
 	 this_class != "PE" ) { 
+      // encountering POS tag
       if ( !previoustag.empty() ) { 
 	waItem.act = previoustag;
 	ana.push_back( waItem );
@@ -306,8 +323,7 @@ vector<waStruct> Mbma::Step1( unsigned int step,
 	  ana.push_back( waItem );
 	  waItem.clear();
 	}
-	previoustag = "i";
-	previoustag += this_class;
+	previoustag = "i" + this_class;
       }
     }
     /* copy the next character */
@@ -325,11 +341,10 @@ vector<waStruct> Mbma::Step1( unsigned int step,
   ana.push_back( waItem );
   waItem.clear();
   
-  /* without changes, but with inflection */
-  if ( this_class.find("/") != string::npos &&
-       this_class != "E/P" ) { 
-    string inflection = "i";
-    inflection += extract_from( this_class, '/' );
+  // do we have inflection?
+  size_t pos = this_class.find( '/' );
+  if ( pos != string::npos && this_class != "E/P" ) { 
+    string inflection = "i" + this_class.substr( pos+1 );
     waItem.act = inflection;
     ana.push_back( waItem );
   }
@@ -381,7 +396,7 @@ void Mbma::resolve_inflections( vector<waStruct>& ana,
     ++step;
     if ( this_class[0]=='i') { 
       if (debugFlag){
-	cout << "thisclass >" << this_class << "<" << endl;
+	*Log(mbmaLog) << "thisclass >" << this_class << "<" << endl;
       }	  
       /* given the specific selections of certain inflections,
 	 change the tag!  */
@@ -392,8 +407,7 @@ void Mbma::resolve_inflections( vector<waStruct>& ana,
 	 the morpheme itself. This is not always the case, but it works  */
       if ( !newtag.empty() ) {
 	if ( debugFlag  ){
-	  cout << "selects " << newtag;
-	  cout << endl;
+	  *Log(mbmaLog) << "selects " << newtag << endl;
 	}
 	/* go left */
 	if ( basictags.find(prev->act[0]) != string::npos ){
@@ -408,38 +422,38 @@ MBMAana Mbma::addInflect( const vector<waStruct>& ana,
 			  const string& inflect, 
 			  const vector<string>& morph ){
   bool found = false;
-  string thisclass;
+  string the_act;
   vector<waStruct>::const_reverse_iterator it = ana.rbegin();
   while ( !found && it != ana.rend() ) { 
     // go back to the last non-inflectional tag 
-    thisclass = it->act;
+    the_act = it->act;
     if (debugFlag){
-      cout << "final tag " << thisclass << endl;
+      *Log(mbmaLog) << "final tag " << the_act << endl;
     }
-    if ( thisclass[0] != 'i' )
+    if ( the_act[0] != 'i' )
       found=true;
     else { 
       ++it;
     }
   }
-  string::size_type pos = thisclass.find_first_of("_/");
+  string::size_type pos = the_act.find_first_of("_/");
   string tag;
   if ( pos != string::npos )
-    tag = thisclass.substr( 0, pos );
+    tag = the_act.substr( 0, pos );
   else 
-    tag = thisclass;
+    tag = the_act;
   string outTag, descr;
   map<string,string>::const_iterator tit = tagNames.find( tag ); 
   if ( tit == tagNames.end() ){
     // unknown tag
     if (debugFlag)
-      cout << "added X 4" << endl;
+      *Log(mbmaLog) << "added X 4" << endl;
     outTag = "X";
     descr = "unknown";
   }
   else {
     if (debugFlag)
-      cout << "added (4) " << tit->second << endl;
+      *Log(mbmaLog) << "added (4) " << tit->second << endl;
     outTag = tit->first;
     descr = tit->second;
   }
@@ -454,41 +468,41 @@ MBMAana Mbma::inflectAndAffix( const vector<waStruct>& ana ){
     if ( !it->word.isEmpty() ){
       morphemes.push_back( UnicodeToUTF8(it->word) );
     }
-    string this_class= it->act;
+    string this_act= it->act;
     if (debugFlag)
-      cout << "unpacking thisclass "<< this_class << endl;
-    if ( inflect.empty() && !this_class.empty() &&
-	 this_class.find("_") == string::npos &&
-	 this_class[0]=='i' ){
-      for ( size_t i=1; i< this_class.length(); ++i ) {
-	if (this_class[i]!='/') {
+      *Log(mbmaLog) << "unpacking act "<< this_act << endl;
+    if ( inflect.empty() && !this_act.empty() &&
+	 this_act.find("_") == string::npos &&
+	 this_act[0]=='i' ){
+      for ( size_t i=1; i< this_act.length(); ++i ) {
+	if (this_act[i]!='/') {
 	  // check if it is a known inflection
-	  map<char,string>::const_iterator csIt = iNames.find(this_class[i]);
+	  map<char,string>::const_iterator csIt = iNames.find(this_act[i]);
 	  if ( csIt == iNames.end() ){
 	    if (debugFlag)
-	      cout << "added X 1" << endl;
+	      *Log(mbmaLog) << "added X 1" << endl;
 	    inflect += "X";
 	  }
 	  else {
 	    if (debugFlag)
-	      cout << "added (1) (" << csIt->first << ") " << csIt->second << endl;
-	    inflect += this_class[i];
+	      *Log(mbmaLog) << "added (1) (" << csIt->first << ") " << csIt->second << endl;
+	    inflect += this_act[i];
 	  }
 	}
       }
       if ( debugFlag )
-	cout << "found inflection " << inflect << endl;
+	*Log(mbmaLog) << "found inflection " << inflect << endl;
     }
     if (debugFlag)	
-      cout << "morphemes now: |" << morphemes << "|" << endl;
+      *Log(mbmaLog) << "morphemes now: |" << morphemes << "|" << endl;
     ++it;
   }
   if (debugFlag)	
-    cout << "inflectAndAffix: " << inflect << " - " << morphemes << endl;
+    *Log(mbmaLog) << "inflectAndAffix: " << inflect << " - " << morphemes << endl;
   if ( inflect.empty() ){
-    MBMAana retVal;
     if (debugFlag)
-      cout << "no Inflection addTag: " << retVal << endl;
+      *Log(mbmaLog) << "Inflection: none " << endl;
+    MBMAana retVal;
     return retVal;
   }
   else {
@@ -496,26 +510,18 @@ MBMAana Mbma::inflectAndAffix( const vector<waStruct>& ana ){
     // tag and stick it at the end
     MBMAana retVal = addInflect( ana, inflect, morphemes );
     if (debugFlag)
-      cout << "Inflection addTag: " << retVal << endl;
+      *Log(mbmaLog) << "Inflection: " << retVal << endl;
     return retVal;
   }
 }
 
-/* "based on mbma.c" is an understatement. The postprocessing is going
-   to be a copy'n'paste from mbma.c for now, since nobody understands
-   the orig code anymore, and there is no time to relearn it now. So
-   it's just going to have a new, and more flexible
-   front-end/wrapper/whatever.
-*/
-
 void Mbma::execute( const UnicodeString& word, 
 		    const vector<string>& classes ){
   const string basictags = "NAQVDOBPYIXZ";
   analysis.clear();
-  /* determine the largest number of alternative analyses 
-     and store every part in a vector of string vectors
-  */
-  int nranal=1;
+  // determine all alternative analyses, remember the largest
+  // and store every part in a vector of string vectors
+  int largest_anal=1;
   vector<vector<string> > classParts;
   classParts.resize( classes.size() );
   for ( unsigned int j=0; j< classes.size(); ++j ){
@@ -523,10 +529,11 @@ void Mbma::execute( const UnicodeString& word,
     int num = split_at( classes[j], parts, "|" );
     if ( num > 0 ){
       classParts[j] = parts;
-      if ( num > nranal )
-	nranal = num;
+      if ( num > largest_anal )
+	largest_anal = num;
     }
     else {
+      // only one, create a dummy
       vector<string> dummy;
       dummy.push_back( classes[j] );
       classParts[j] = dummy;
@@ -534,53 +541,46 @@ void Mbma::execute( const UnicodeString& word,
   }
   
   if (debugFlag){
-    cout << "mbma_bb, word=" << word << ", classes=<";
+    string out = "alternatives: word=" + UnicodeToUTF8(word) + ", classes=<";
     for ( size_t i=0; i < classes.size(); ++i )
-      cout << classes[i] << ",";
-    cout << ">" << endl;
+      out += classes[i] + ",";
+    out += ">";
+    *Log(mbmaLog) << out << endl;
   }    
-  /* then for each analysis, produce a labeled bracketed string */
   
-  /* produce a basic bracketed string, taking care of
-     insertions, deletions, and replacements */
-  
-  for ( unsigned int step=nranal; step> 0; --step ) { 
-    vector<waStruct> ana = Step1( step, word, nranal, 
+  // now loop over all the analysis
+  for ( unsigned int step=largest_anal; step> 0; --step ) { 
+    vector<waStruct> ana = Step1( step, word, largest_anal, 
 				  classParts, basictags );
     if (debugFlag)
-      cout << "intermediate analysis 1: " << ana << endl;
+      *Log(mbmaLog) << "intermediate analysis 1: " << ana << endl;
     resolve_inflections( ana, basictags );
     /* finally, unpack the tag and inflection names to make everything readable */
     if (debugFlag)
-      cout << "intermediate analysis 3: " << ana << endl;
+      *Log(mbmaLog) << "intermediate analysis 3: " << ana << endl;
     MBMAana tmp = inflectAndAffix( ana );
-    if (debugFlag){
-      cout << "b_out: " << endl;
-      cout << tmp << endl;
-    }
     analysis.push_back( tmp );
   }
 }
 
-void Mbma::addMorph( FoliaElement *word, 
+void Mbma::addMorph( Word *word, 
 		     const vector<string>& lemmas ){
-  FoliaElement *ml = new MorphologyLayer("");
+  MorphologyLayer *ml = new MorphologyLayer();
 #pragma omp critical(foliaupdate)
   {
     word->append( ml );
   }
   int offset = 0;
-  string args = "annotator='mbma'";
   for ( size_t p=0; p < lemmas.size(); ++p ){
-    FoliaElement *m = new Morpheme("");
+    Morpheme *m = new Morpheme();
 #pragma omp critical(foliaupdate)
     {
       ml->append( m );
     }
-    FoliaElement *t = 
-      new TextContent( "value='" + escape( lemmas[p]) + 
-			      "', offset='" + toString(offset) + "'" );
-
+    KWargs args;
+    args["value"] = lemmas[p];
+    args["offset"] = toString(offset);
+    TextContent *t = new TextContent( args );
     offset += lemmas[p].length();
 #pragma omp critical(foliaupdate)
     {
@@ -589,18 +589,15 @@ void Mbma::addMorph( FoliaElement *word,
   }
 }	  
       
-void Mbma::postprocess( FoliaElement *fword, PosAnnotation *pos ){
+void Mbma::postprocess( Word *fword, PosAnnotation *pos ){
   if (debugFlag){
     for(vector<MBMAana>::const_iterator it=analysis.begin(); it != analysis.end(); it++)
-      cout << it->getTag() << it->getInflection()<< " ";
-    cout << "\n\tmorph analyses: ";
+      *Log(mbmaLog) << "tag " <<  it->getTag() + " inflection: " << it->getInflection() << endl;
+    *Log(mbmaLog) << "morph analyses: " << endl;;
     for (vector<MBMAana>::const_iterator it=analysis.begin(); it != analysis.end(); it++)
-      cout << it->getMorph() << " ";
-    cout << endl <<endl;
+      *Log(mbmaLog) << it->getMorph() << endl;
+    *Log(mbmaLog) << "before morpho: " << endl;
   }
-  
-  if (debugFlag)
-      cout << "before morpho: " << endl;
   const string tag = pos->feat("head");
   map<string,string>::const_iterator tagIt = TAGconv.find( tag );
   if ( tagIt == TAGconv.end() ) {
@@ -608,30 +605,30 @@ void Mbma::postprocess( FoliaElement *fword, PosAnnotation *pos ){
     // this should never happen
     //
     if (debugFlag){
-      cout << "no match!\n";
+      *Log(mbmaLog) << "no match!" << endl;
     } 
     throw ValueError( "unknown pos tag value '" + tag + "'" );
   }
   else {
     if (debugFlag){
-      cout << "#matches: " << tagIt->first << " " << tagIt->second << endl;
+      *Log(mbmaLog) << "#matches: " << tagIt->first << " " << tagIt->second << endl;
     }
     size_t match = 0;	
     vector<size_t> matches;
     for ( size_t m_index = 0; m_index <analysis.size(); ++m_index ) {
       if (debugFlag)
-	cout << "comparing " << tagIt->second << " with " << analysis[m_index].getTag() << endl;
+	*Log(mbmaLog) << "comparing " << tagIt->second << " with " << analysis[m_index].getTag() << endl;
       if ( tagIt->second == analysis[m_index].getTag() ) {
 	match++;
 	matches.push_back( m_index );
       }
     }
     if (debugFlag) {
-      cout << "main tag " << tagIt->second
-	   << " matches " << match 
-	   << " morpho analyses: " << endl;
+      *Log(mbmaLog) << "main tag " << tagIt->second
+		    << " matches " << match 
+		    << " morpho analyses: " << endl;
       for( size_t p=0; p < match; ++p )  {
-	cout << "\tmatch #" << p << " : " << analysis[matches[p]].getMorph() << endl;
+	*Log(mbmaLog) << "\tmatch #" << p << " : " << analysis[matches[p]].getMorph() << endl;
       }
     }
     //should be(come?) a switch
@@ -652,15 +649,8 @@ void Mbma::postprocess( FoliaElement *fword, PosAnnotation *pos ){
       addMorph( fword, ma );
     } 
     else {
-      vector<Feature*> feats = pos->select<Feature>();
-      if (debugFlag){
-	cout << "tag: " << tag << endl;
-	for ( size_t q =0 ; q < feats.size(); ++q ) {
-	  cout << "\tpart #" << q << ": " << feats[q]->cls() << endl;
-	}
-      }
       if (debugFlag)
-	cout << "multiple lemma's\n";
+	*Log(mbmaLog) << "multiple lemma's" << endl;
       map<string, int> possible_lemmas;
       // find unique lemma's
       for ( size_t q = 0; q < match; ++q ) {
@@ -677,23 +667,30 @@ void Mbma::postprocess( FoliaElement *fword, PosAnnotation *pos ){
 	}
       }
       if (debugFlag)
-	cout << "#unique lemma's: " << possible_lemmas.size() << endl;
+	*Log(mbmaLog) << "#unique lemma's: " << possible_lemmas.size() << endl;
       if ( possible_lemmas.size() < 1) {
 	if (debugFlag)
-	  cout << "Problem!, no possible lemma's" << endl;
+	  *Log(mbmaLog) << "Problem!, no possible lemma's" << endl;
 	return;
       }
       // append all unique lemmas
       // find best match
       // loop through all subParts of the tag
       // and match with inflections from each m
+      vector<Feature*> feats = pos->select<Feature>();
+      if (debugFlag){
+	*Log(mbmaLog) << "tag: " << tag << endl;
+	for ( size_t q =0 ; q < feats.size(); ++q ) {
+	  *Log(mbmaLog) << "\tpart #" << q << ": " << feats[q]->cls() << endl;
+	}
+      }
       map <string, const vector<string>* > bestMatches;
       int max_count = 0;
       for ( size_t q=0; q < match; ++q ) {
 	int match_count = 0;
 	string inflection = analysis[matches[q]].getInflection();
 	if (debugFlag)
-	  cout << "matching " << inflection << " with " << tag << endl;
+	  *Log(mbmaLog) << "matching " << inflection << " with " << tag << endl;
 	for ( size_t u=0; u < feats.size(); ++u ) {
 	  map<string,string>::const_iterator conv_tag_p = TAGconv.find(feats[u]->cls());
 	  if (conv_tag_p != TAGconv.end()) {
@@ -703,7 +700,7 @@ void Mbma::postprocess( FoliaElement *fword, PosAnnotation *pos ){
 	  }
 	}
 	if (debugFlag)
-	  cout << "score: " << match_count << " max was " << max_count << endl;
+	  *Log(mbmaLog) << "score: " << match_count << " max was " << max_count << endl;
 	if (match_count >= max_count) {
 	  string key;
 	  const vector<string> *ma = &analysis[matches[q]].getMorph();
@@ -723,7 +720,7 @@ void Mbma::postprocess( FoliaElement *fword, PosAnnotation *pos ){
 	  string tmp;
 	  for ( size_t p=0; p < it->second->size(); ++p )
 	    tmp += "[" + (*it->second)[p] + "]";
-	  cout << "MBMA add lemma: " << tmp << endl;
+	  *Log(mbmaLog) << "MBMA add lemma: " << tmp << endl;
 	}
 	addMorph( fword, *it->second );
 	++it;
@@ -732,7 +729,13 @@ void Mbma::postprocess( FoliaElement *fword, PosAnnotation *pos ){
   }
 }  // postprocess
 
-bool Mbma::Classify( FoliaElement* sword ){
+void Mbma::addDeclaration( Document& doc ) const {
+  doc.declare( AnnotationType::MORPHOLOGICAL, tagset,
+	       "annotator='frog-mbma-" +  version + 
+	       "', annotatortype='auto'");
+}
+  
+bool Mbma::Classify( Word* sword ){
   UnicodeString uWord;
   PosAnnotation *pos;
 #pragma omp critical(foliaupdate)
@@ -741,7 +744,7 @@ bool Mbma::Classify( FoliaElement* sword ){
     pos = sword->annotation<PosAnnotation>();
   }
   string tag = pos->feat("head");
-  if ( tag == "SPEC" ){
+  if ( tag == "SPEC" || tag == "LET" ){
     string word = UnicodeToUTF8( uWord );
     vector<string> tmp;
     tmp.push_back( word );
@@ -750,33 +753,29 @@ bool Mbma::Classify( FoliaElement* sword ){
   }
   uWord.toLower();
   if (debugFlag)
-    cout << "Classify word: " << uWord << endl;
+    *Log(mbmaLog) << "Classify word: " << uWord << endl;
   
   vector<string> insts = make_instances( uWord );
   vector<string> classes;
   for( size_t i=0; i < insts.size(); ++i ) {
     string ans;
-    string db;
-    double d;
-    MTree->Classify( insts[i], ans, db, d );
+    MTree->Classify( insts[i], ans );
     if ( debugFlag )
-      cerr << "itt #" << i << ": timbl gave class= " << ans 
-	   << " from distrib= " << db << endl; 
+      *Log(mbmaLog) << "itt #" << i << ": timbl gave class= " << ans << endl; 
     classes.push_back( ans);
   }
+
   // fix for 1st char class ==0
   if ( classes[0] == "0" )
-    classes[0] = "X";
-  
+    classes[0] = "X";  
   execute( uWord, classes );
   postprocess( sword, pos );
   return true;
 }
 
 ostream& operator<< ( ostream& os, const MBMAana& a ){
-  os << a.tag << " " 
-     << a.infl << " >>>> " << a.morphemes << " <<<< [[[[ "
-     << a.description << " ]]]] ";
+  os << "tag: " << a.tag << " infl:" << a.infl << " morhemes: " 
+     << a.morphemes << " description: " << a.description;
   return os;
 }
 
