@@ -1,5 +1,5 @@
 /*
-  $Id: mwu_chunker_mod.cxx 13840 2012-01-02 13:47:26Z sloot $
+  $Id: mwu_chunker_mod.cxx 14478 2012-03-19 16:10:38Z sloot $
   $URL: https://ilk.uvt.nl/svn/sources/Frog/trunk/src/mwu_chunker_mod.cxx $
 
   Copyright (c) 2006 - 2012
@@ -37,20 +37,27 @@
 #include "timbl/TimblAPI.h"
 
 #include "frog/Frog.h" // defines etc.
-#include "libfolia/folia.h" // defines etc.
+#include "libfolia/folia.h" 
+#include "libfolia/document.h"
 #include "frog/mwu_chunker_mod.h"
 
 using namespace folia;
 using namespace std;
 
-mwuAna::mwuAna( FoliaElement *fwrd ){
+mwuAna::mwuAna( Word *fwrd ){
   spec = false;
-  //  fword = fwrd;
   word = fwrd->str();
-  string tag = fwrd->annotation<PosAnnotation>()->feat("head");
+  string tag;
+#pragma omp critical(foliaupdate)
+  {
+    tag = fwrd->annotation<PosAnnotation>()->feat("head");
+  }
   if ( tag == "SPEC" ){
-    vector<Feature*> feats = fwrd->select<Feature>();
-    spec = ( feats.size() == 1 && feats[0]->cls() == "deeleigen" );
+#pragma omp critical(foliaupdate)
+    {
+      vector<Feature*> feats = fwrd->select<Feature>();
+      spec = ( feats.size() == 1 && feats[0]->cls() == "deeleigen" );
+    }
   }
   fwords.push_back( fwrd );
 }  
@@ -60,22 +67,22 @@ void mwuAna::merge( const mwuAna *add ){
   delete add;
 }
 
-void mwuAna::addEntity( FoliaElement *sent ){
+EntitiesLayer *mwuAna::addEntity( const std::string& tagset,
+				  Sentence *sent, EntitiesLayer *el ){
   if ( fwords.size() > 1 ){
-    FoliaElement *el = 0;
-    try {
-      el = sent->annotation<EntitiesLayer>( );
-    }
-    catch(...){
-      el = new EntitiesLayer("");
+    if ( el == 0 ){
 #pragma omp critical(foliaupdate)
       {
+	el = new EntitiesLayer();
 	sent->append( el );
       }
     }
-    FoliaElement *e = new Entity("");
+    KWargs args;
+    args["set"] = tagset;
+    Entity *e=0;
 #pragma omp critical(foliaupdate)
     {
+      e = new Entity( el->doc(), args );
       el->append( e );
     }
     for ( size_t p=0; p < fwords.size(); ++p ){
@@ -85,9 +92,17 @@ void mwuAna::addEntity( FoliaElement *sent ){
       }
     }
   }
+  return el;
 }
 
-Mwu::~Mwu(){ reset(); }
+Mwu::Mwu(){
+  mwuLog = new LogStream( theErrLog, "mwu-" );
+}
+
+Mwu::~Mwu(){ 
+  reset();
+  delete mwuLog;
+}
 
 void Mwu::reset(){
   for( size_t i=0; i< mWords.size(); ++i ){
@@ -96,13 +111,13 @@ void Mwu::reset(){
   mWords.clear();
 }
 
-void Mwu::add( FoliaElement *word ){
+void Mwu::add( Word *word ){
   mWords.push_back( new mwuAna( word ) );
 }
 
 
 bool Mwu::read_mwus( const string& fname) {
-  *Log(theErrLog) << "read mwus " + fname + "\n";
+  *Log(mwuLog) << "read mwus " + fname << endl;
   ifstream mwufile(fname.c_str(), ios::in);
   if(mwufile.bad()){
     return false;
@@ -117,7 +132,7 @@ bool Mwu::read_mwus( const string& fname) {
       MWUs.insert( make_pair( key, res2 ) );
     }
     else {
-      *Log(theErrLog) << "invalid entry in MWU file " << line << endl;
+      *Log(mwuLog) << "invalid entry in MWU file " << line << endl;
       return false;
     }
   }
@@ -125,25 +140,33 @@ bool Mwu::read_mwus( const string& fname) {
 }
   
 bool Mwu::init( const Configuration& config ) {
-  myCFS = "_";
-  *Log(theErrLog) << "initiating mwuChunker ... \n";
+  *Log(mwuLog) << "initiating mwuChunker..." << endl;
   debug = tpDebug;
-  string db = config.lookUp( "debug", "mwu" );
-  if ( !db.empty() )
-    debug = stringTo<int>( db );
-  string att = config.lookUp( "t", "mwu" );  
-  if ( att.empty() ){
-    *Log(theErrLog) << "cannot find attribute 't' in configfile" << endl;
+  string val = config.lookUp( "debug", "mwu" );
+  if ( !val.empty() )
+    debug = stringTo<int>( val );
+  val = config.lookUp( "t", "mwu" );  
+  if ( val.empty() ){
+    *Log(mwuLog) << "cannot find attribute 't' in configfile" << endl;
     return false;
   }
-  mwuFileName = prefix( config.configDir(), att );
-  att = config.lookUp( "c", "mwu" );  
-  if ( !att.empty() )
-    myCFS = att;
-  if (!read_mwus(mwuFileName)) {
-    *Log(theErrLog) << "Cannot read mwu file " << mwuFileName << endl;
+  mwuFileName = prefix( config.configDir(), val );
+  if ( !read_mwus(mwuFileName) ) {
+    *Log(mwuLog) << "Cannot read mwu file " << mwuFileName << endl;
     return false;
   }
+  val = config.lookUp( "version", "mwu" );
+  if ( val.empty() ){
+    version = "1.0";
+  }
+  else
+    version = val;
+  val = config.lookUp( "set", "mwu" );
+  if ( val.empty() ){
+    tagset = "http://ilk.uvt.nl/folia/sets/frog-mwu-nl";
+  }
+  else
+    tagset = val;
   return true;
 }
 
@@ -154,20 +177,32 @@ ostream &operator <<( ostream& os,
   return os;
 }
 
-void Mwu::Classify( FoliaElement *sent ){
+void Mwu::addDeclaration( Document& doc ) const {
+  doc.declare( AnnotationType::ENTITY, 
+	       tagset,
+	       "annotator='frog-mwu-" + version
+	       + "', annotatortype='auto'");
+}
+
+void Mwu::Classify( Sentence *sent ){
   reset();
-  vector<Word*> words = sent->words();
+  vector<Word*> words;
+#pragma omp critical(foliaupdate)
+  {
+    words = sent->words();
+  }
   for ( size_t i=0; i < words.size(); ++i )
     add( words[i] );  
   Classify();
+  EntitiesLayer *el = 0;
   for( size_t i = 0; i < mWords.size(); ++i ){
-    mWords[i]->addEntity( sent );
+    el = mWords[i]->addEntity( tagset, sent, el );
   }
 }
 
 void Mwu::Classify(){
   if ( debug )
-    cout << "\nStarting mwu Classify\n";
+    *Log(mwuLog) << "Starting mwu Classify" << endl;
   mymap2::iterator best_match;
   size_t matchLength = 0;
   size_t max = mWords.size();
@@ -189,31 +224,31 @@ void Mwu::Classify(){
   for( i = 0; i < max; i++) {
     string word = mWords[i]->getWord();
     if ( debug )
-      cout << "checking word[" << i <<"]: " << word << endl;
+      *Log(mwuLog) << "checking word[" << i <<"]: " << word << endl;
     pair<mymap2::iterator, mymap2::iterator> matches = MWUs.equal_range(word);
     if ( matches.first != MWUs.end() ) {
       //match
       mymap2::iterator current_match = matches.first;
       if (  debug  ) {
-	cout << "MWU: match found!\t" << current_match->first << endl;
+	*Log(mwuLog) << "MWU: match found!\t" << current_match->first << endl;
       }
       while(current_match != matches.second && current_match != MWUs.end()) {
 	vector<string> match = current_match->second;
 	size_t max_match = match.size();
 	size_t j = 0;
 	if ( debug )
-	  cout << "checking " << max_match << " matches\n";
+	  *Log(mwuLog) << "checking " << max_match << " matches:" << endl;
 	for(; i + j + 1 < max && j < max_match; j++) {
 	  if ( match[j] != mWords[i+j+1]->getWord() ) {
 	    if ( debug )
-	      cout << "match " << j <<" (" << match[j] 
+	      *Log(mwuLog) << "match " << j <<" (" << match[j] 
 		   << ") doesn't match with word " << i+ j + 1
-		   << " (" << mWords[i+j + 1]->getWord() <<")\n";
+			   << " (" << mWords[i+j + 1]->getWord() <<")" << endl;
 	    // mismatch in jth word of current mwu
 	    break;
 	  }
 	  else if ( debug )
-	    cout << " matched " <<  mWords[i+j+1]->getWord() << " j=" << j << endl;
+	    *Log(mwuLog) << " matched " <<  mWords[i+j+1]->getWord() << " j=" << j << endl;
 	  
 	}
 	if (j == max_match && j > matchLength ){
@@ -225,9 +260,9 @@ void Mwu::Classify(){
       } // while
       if( debug ){
 	if (matchLength >0 ) {
-	  cout << "MWU: found match starting with " << (*best_match).first << endl;
+	  *Log(mwuLog) << "MWU: found match starting with " << (*best_match).first << endl;
 	} else {
-	  cout <<"MWU: no match\n";
+	  *Log(mwuLog) <<"MWU: no match" << endl;
 	}
       }
       // we found a matching mwu, break out of loop thru sentence, 
@@ -237,24 +272,24 @@ void Mwu::Classify(){
     } //match found
     else { 
       if( debug )
-	cout <<"MWU:check: no match\n";
+	*Log(mwuLog) <<"MWU:check: no match" << endl;
     }
   } //for (i < max)
   if (matchLength > 0 ) {
     //concat
     if ( debug )
-      cout << "mwu found, processing\n";
+      *Log(mwuLog) << "mwu found, processing" << endl;
     for ( size_t j = 1; j <= matchLength; ++j) {
       if ( debug )
-	cout << "concat " << mWords[i+j]->getWord() << endl;
+	*Log(mwuLog) << "concat " << mWords[i+j]->getWord() << endl;
       mWords[i]->merge( mWords[i+j] );
     }
     vector<mwuAna*>::iterator anatmp1 = mWords.begin() + i;
     vector<mwuAna*>::iterator anatmp2 = ++anatmp1 + matchLength;
     mWords.erase(anatmp1, anatmp2);
     if ( debug ){
-      cout << "tussenstand:" << endl;
-      cout << *this << endl;
+      *Log(mwuLog) << "tussenstand:" << endl;
+      *Log(mwuLog) << *this << endl;
     }      
     Classify( );
   } //if (matchLength)
